/*
 * Copyright (c) 2019-2021 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Sun Jun 30 2019
 */
#ifndef H_TK3_SYS
#define H_TK3_SYS

#include "hal.h"
#include "osal.h"


/* --- events -------------------------------------------------------------- */

static inline void
tk3ev_signal()
{
  SCB->ICSR |= (1 << SCB_ICSR_PENDSVSET_Pos);
}


/* --- locking ------------------------------------------------------------- */

static inline uint32_t
syslock(void)
{
  uint32_t pri = __get_BASEPRI();
  __set_BASEPRI(OSAL_BASEPRI(OSAL_IRQ_MAXIMUM_PRIORITY));
  return pri;
}

static inline uint32_t
sysunlock(void)
{
  uint32_t pri = __get_BASEPRI();
  __set_BASEPRI(0);
  return pri;
}

static inline void
syslock_restore(uint32_t *pri)
{
  __set_BASEPRI(*pri);
}

#define with_syslock()                                                  \
  for(uint32_t basepri                                                  \
        __attribute__((__cleanup__(syslock_restore))) = syslock(),      \
        do_ = 1; do_; do_ = 0)

#define without_syslock()                                               \
  for(uint32_t basepri                                                  \
        __attribute__((__cleanup__(syslock_restore))) = sysunlock(),    \
        do_ = 1; do_; do_ = 0)


#endif /* H_TK3_SYS */
